<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Block\Adminhtml;

use Magento\Framework\Data\Form\Element\CollectionFactory;
use Magento\Framework\Data\Form\Element\Factory;
use Magento\Framework\Data\Form\Element\Multiselect;
use Magento\Framework\Escaper;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Math\Random;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Helper\SecureHtmlRenderer;
use Magento\Store\Model\StoreManagerInterface;

class Provider extends MultiSelect
{
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @param StoreManagerInterface $storeManager
     * @param Factory $factoryElement
     * @param CollectionFactory $factoryCollection
     * @param Escaper $escaper
     * @param array $data
     * @param SecureHtmlRenderer|null $secureRenderer
     * @param Random|null $random
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        Factory               $factoryElement,
        CollectionFactory     $factoryCollection,
        Escaper               $escaper,
        array                 $data = [],
        ?SecureHtmlRenderer   $secureRenderer = null,
        ?Random               $random = null
    )
    {
        $this->storeManager = $storeManager;
        parent::__construct($factoryElement, $factoryCollection, $escaper, $data, $secureRenderer, $random);
    }

    /**
     * @return string
     * @throws NoSuchEntityException
     */
    public function getElementHtml(): string
    {
        $this->setCanBeEmpty(true);
        return parent::getElementHtml() . $this->getJs();
    }

    /**
     * @return string
     * @throws NoSuchEntityException
     */
    protected function getJs(): string
    {
        $url = $this->storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_WEB, true) . 'connectpay/provider/index';
        return "
                <script type='text/javascript'>
    require(['jquery', 'mage/translate'], function ($) {
    let selectCountry = $('[id$=_connectpay_payment_country]').find('select');
    let selectProvider = $('[id$=_connectpay_provider_id]').find('select');
    $(selectCountry).on('change', getProvider);

    function getProvider() {
        if (selectCountry.children('option:selected').val() === 'DE') {
            return;
        }
        $.ajax({
            url: '{$url}',
            type: 'POST',
            data: {
                countryCode: selectCountry.children('option:selected').val(),
            },
            dataType: 'json',
            showLoader: true,
            success: function (result) {
                selectProvider.empty();
                if (result !== null) {
                    result.forEach(function (element) {
                        selectProvider.append('<option value=' + element.id + '>' + element.name + '</option>');
                    });
                } else {
                    alert($.mage.__('Could not get data from providers'));
                    selectProvider.empty();
                }
            },
            error: function () {
                alert($.mage.__('Something went wrong'));
                selectProvider.empty();
            },
        })
    }
})
</script>
";
    }
}
