<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Controller\Provider;

use ConnectPay\Payment\Gateway\Config\Config;
use ConnectPay\Payment\Model\ProviderMapper;
use ConnectPay\Payment\Model\Request\Provider;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Exception\LocalizedException;

class Index implements HttpPostActionInterface, CsrfAwareActionInterface
{
    /**
     * @var JsonFactory
     */
    private $resultJsonFactory;

    /**
     * @var RequestInterface
     */
    private $request;

    /**
     * @var Provider
     */
    private $provider;

    /**
     * @var ProviderMapper
     */
    private $mapper;

    /**
     * @param JsonFactory $resultJsonFactory
     * @param RequestInterface $request
     * @param Provider $provider
     * @param ProviderMapper $mapper
     */
    public function __construct(
        JsonFactory      $resultJsonFactory,
        RequestInterface $request,
        Provider         $provider,
        ProviderMapper   $mapper
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->request = $request;
        $this->provider = $provider;
        $this->mapper = $mapper;
    }

    /**
     * @return Json
     * @throws LocalizedException
     */
    public function execute(): Json
    {
        $countryCode = $this->request->getParam(Config::KEY_COUNTRY_CODE);
        $providersData = $this->provider->getProviders($countryCode);
        $providers = $this->mapper->getProviderData($providersData);
        $result = $this->resultJsonFactory->create();
        if (!empty($providers)) {
            $result->setData($providers);
        }
        return $result;
    }

    /**
     * @param RequestInterface $request
     * @return bool|null
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }

    /**
     * @param RequestInterface $request
     * @return InvalidRequestException|null
     */
    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }
}
