<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Controller\Redirect;

use ConnectPay\Payment\Gateway\Config\Config;
use Exception;
use Magento\Checkout\Model\Session;
use Magento\Checkout\Model\Session\SuccessValidator;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Controller\Result\RedirectFactory;
use Magento\Framework\Message\ManagerInterface;
use Magento\Payment\Gateway\Command\CommandPoolInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectFactory;
use Magento\Sales\Api\Data\OrderPaymentInterface;

class Index implements HttpGetActionInterface
{
    /**
     * @var RedirectFactory
     */
    private $redirectFactory;

    /**
     * @var SuccessValidator
     */
    private $successValidator;

    /**
     * @var Session
     */
    private $checkoutSession;

    /**
     * @var ManagerInterface
     */
    private $messageManager;

    /**
     * @var CommandPoolInterface
     */
    private $commandPool;

    /**
     * @var PaymentDataObjectFactory
     */
    private $paymentDataObjectFactory;

    public function __construct(
        RedirectFactory          $redirectFactory,
        SuccessValidator         $successValidator,
        Session                  $checkoutSession,
        ManagerInterface         $messageManager,
        CommandPoolInterface     $commandPool,
        PaymentDataObjectFactory $paymentDataObjectFactory
    )
    {
        $this->redirectFactory = $redirectFactory;
        $this->successValidator = $successValidator;
        $this->checkoutSession = $checkoutSession;
        $this->messageManager = $messageManager;
        $this->commandPool = $commandPool;
        $this->paymentDataObjectFactory = $paymentDataObjectFactory;
    }

    /**
     * @return Redirect
     * @throws Exception
     */
    public function execute(): Redirect
    {
        $resultRedirect = $this->redirectFactory->create();
        $isValid = $this->successValidator->isValid();
        if ($isValid) {
            $order = $this->checkoutSession->getLastRealOrder();
            $orderPayment = $order->getPayment();
            if (!$orderPayment instanceof OrderPaymentInterface) {
                throw new Exception('Order Payment does not exist.');
            }

            $isValid = ($orderPayment->getMethod() === Config::CODE);
        }

        if (!$isValid) {
            $this->messageManager->addErrorMessage(
                __('Can\'t find order.')
            );
            return $resultRedirect->setPath('checkout/cart');
        }

        $data = [
            'order' => $order,
            'payment' => $this->paymentDataObjectFactory->create($orderPayment)
        ];

        $result = $this->commandPool->get(Config::REQUEST_COMMAND)->execute($data);
        if (is_null($result)) {
            $this->messageManager->addErrorMessage(
                __('Something went wrong while processing request')
            );
            return $resultRedirect->setPath('checkout/cart');
        }
        $redirectUrl = $result->get();
        return $resultRedirect->setPath($redirectUrl);
    }
}
