<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway\Command;

use ConnectPay\Payment\Gateway\Config\Config;
use ConnectPay\Payment\Gateway\Http\Client\Guzzle;
use ConnectPay\Payment\Gateway\SubjectReader;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\Webapi\Rest\Request;
use Magento\Payment\Gateway\CommandInterface;
use Magento\Payment\Gateway\Http\TransferFactoryInterface;
use Magento\Payment\Gateway\Response\HandlerInterface;
use Magento\Sales\Api\OrderRepositoryInterface as OrderRepository;

class CallbackCommand implements CommandInterface
{
    /**
     * @var TransferFactoryInterface
     */
    private $transferFactory;

    /**
     * @var Guzzle
     */
    private $client;

    /**
     * @var HandlerInterface
     */
    private $handler;

    /**
     * @var Json
     */
    private $serializer;

    /**
     * @var SubjectReader
     */
    private $subjectReader;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    public function __construct(
        OrderRepository          $orderRepository,
        TransferFactoryInterface $transferFactory,
        Guzzle                   $client,
        Json                     $serializer,
        SubjectReader            $subjectReader,
        HandlerInterface         $handler = null
    )
    {
        $this->orderRepository = $orderRepository;
        $this->transferFactory = $transferFactory;
        $this->client = $client;
        $this->serializer = $serializer;
        $this->subjectReader = $subjectReader;
        $this->handler = $handler;
    }

    public function execute(array $commandSubject)
    {
        $data = [];
        $transferObject = $this->transferFactory->create($data);
        $data = $transferObject->getBody();
        $paymentDO = $this->subjectReader->readPayment($commandSubject);
        $orderId = $paymentDO->getOrder()->getId();
        $order = $this->orderRepository->get($orderId);
        $payment = $order->getPayment();
        $cpPaymentId = $payment->getData(Config::KEY_PAYMENT_ID);
        if(!is_null($cpPaymentId))
        {
            $response = $this->client->doRequest($data, Request::HTTP_METHOD_GET, Config::API_REQUEST_INITIATE_PAYMENT_ENDPOINT . '/' . $cpPaymentId);
            $responseArray = $this->serializer->unserialize($response->getBody()->getContents());

            if ($this->handler) {
                $this->handler->handle(
                    $commandSubject,
                    $responseArray
                );
            }
        }

    }
}
