<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway\Config;

interface ConfigInterface
{
    /**
     * Payment method code
     *
     * @var string
     */
    const CODE = 'connectpay';

    /**
     * List of supported languages
     *
     * @var array
     */
    const LANGUAGES_CODES_ARRAY = ['EN', 'FI', 'NL', 'DE', 'SV', 'LT'];

    /**
     * List of supported countries
     *
     * @var array
     */
    const COUNTRY_CODES_ARRAY = ['FI', 'NL', 'LT', 'DE'];

    /**
     * Redirect path
     *
     * @var string
     */
    const REDIRECT_URL = 'connectpay/redirect';

    const NOTIFICATION_URL = 'connectpay/callback';

    const KEY_MERCHANT = 'merchant';

    const KEY_PAYMENT = 'payment';

    const KEY_PAYMENT_ID = 'connectpay_payment_id';

    const KEY_STATUS_IS_TERMINAL = 'isTerminal';

    const KEY_STATUS_CODE = 'code';

    const KEY_BRAND_ID = 'brandId';

    const XML_PATH_BRAND_ID = 'brand_id';

    const KEY_INSTRUCTED_AMOUNT = 'instructedAmount';

    const KEY_AMOUNT = 'amount';

    const KEY_CURRENCY = 'currency';

    const KEY_REMITTANCE_INFORMATION_UNSTRUCTURED = 'remittanceInformationUnstructured';

    const KEY_PROVIDER_COUNTRY_CODE = 'providerCountryCode';

    const KEY_PROVIDER_ID = 'providerId';

    const XML_PATH_PROVIDER_COUNTRY_CODE = 'payment_country';

    const XML_PATH_PROVIDER_ID = 'provider_id';

    const PAYMENT_METHOD = 'PIS';

    const KEY_PAYMENT_METHOD = 'paymentMethod';

    const KEY_CONSUMER = 'consumer';

    const KEY_FIRST_NAME = 'firstName';

    const KEY_LAST_NAME = 'lastName';

    const KEY_CONSUMER_ID = 'consumerId';

    const KEY_IP_ADDRESS = 'ipAddress';

    const KEY_PHONE_NUMBER = 'phoneNo';

    const KEY_WIDGET_LANGUAGE = 'language';

    const KEY_REDIRECT_URL = 'redirectUrl';

    const KEY_NOTIFICATION_URL = 'notificationUrl';

    const KEY_EMAIL = 'email';

    const KEY_IDENTIFIERS = 'identifiers';

    const KEY_MERCHANT_REFERENCE = 'merchantReference';

    const KEY_END_TO_END_ID = 'endToEndId';

    const KEY_COUNTRY_CODE = 'countryCode';

    const XML_PATH_WIDGET_LANGUAGE = 'widget_language';

    const XML_PATH_PRIVATE_KEY = 'private_key';

    const XML_PATH_CLIENT_ID = 'client_id';

    const XML_PATH_CLIENT_SECRET = 'client_secret';

    const XML_PATH_SANDBOX_MODE = 'test';

    const CONNECTPAY_DIR = 'connectpay';

    const PAYMENT_CODE = 'connectpay';

    const API_REQUEST_URI_STAGE = 'https://api2-stage.connectpay.com/';

    const API_REQUEST_URI_PROD = 'https://api2.connectpay.com/';

    const API_REQUEST_INITIATE_PAYMENT_ENDPOINT = 'merchant/payments';

    const API_REQUEST_GET_PROVIDERS_ENDPOINT = 'merchant/providers';

    const REQUEST_COMMAND = 'request';

    const CALLBACK_COMMAND = 'callback';

    const APPROVED_STATUS = 'Approved';

    public function getBrandId(): string;

    public function getWidgetLanguage(): string;

    public function getProviderCountryCode(): string;

    public function getProviderId(): array;

    public function getPrivateKey(): string;

    public function getClientId(): string;

    public function getClientSecret(): string;

    public function getUuid(): string;

    public function getRedirectUrl(): string;

    public function isTestModeEnabled(): bool;
}
