<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway\Http\Client;

use ConnectPay\Payment\Gateway\Config\Config;
use GuzzleHttp\ClientFactory;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Psr7\ResponseFactory;
use Psr\Log\LoggerInterface as Logger;

class Guzzle
{
    /**
     * @var ClientFactory
     */
    private $clientFactory;

    /**
     * @var ResponseFactory
     */
    private $responseFactory;

    /**
     * @var Logger
     */
    private $logger;

    /**
     * @var Config
     */
    private $config;

    /**
     * @param ClientFactory $clientFactory
     * @param ResponseFactory $responseFactory
     * @param Logger $logger
     * @param Config $config
     */
    public function __construct(
        ClientFactory   $clientFactory,
        ResponseFactory $responseFactory,
        Logger          $logger,
        Config          $config
    )
    {
        $this->clientFactory = $clientFactory;
        $this->responseFactory = $responseFactory;
        $this->logger = $logger;
        $this->config = $config;
    }

    /**
     * @param array $data
     * @param string $requestMethod
     * @param string $uriEndpoint
     * @return Response
     */
    public function doRequest(
        array  $data,
        string $requestMethod,
        string $uriEndpoint
    ): Response
    {
        if ($this->config->isTestModeEnabled()) {
            $url = Config::API_REQUEST_URI_STAGE;
        } else {
            $url = Config::API_REQUEST_URI_PROD;
        }

        $client = $this->clientFactory->create(
            ['config' => [
                'base_uri' => $url,
                'auth' => [$this->config->getClientId(), $this->config->getClientSecret()],
                'headers' => [
                    'X-Signature' => $this->config->getSignature($data, $requestMethod, $url, $uriEndpoint),
                ],
            ]]
        );
        try {
            $response = $client->request(
                $requestMethod,
                $uriEndpoint,
                $data
            );

        } catch (GuzzleException $exception) {
            $responseMessage = $exception->getResponse();
            $responseMessage = !is_null($responseMessage) ? $responseMessage->getBody() : $exception->getMessage();
            $this->logger->critical($responseMessage);
            $response = $this->responseFactory->create([
                'status' => $exception->getCode(),
                'reason' => $exception->getMessage()
            ]);
        }

        return $response;
    }
}
