<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway\Http;

use ConnectPay\Payment\Gateway\Config\Config;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Payment\Gateway\Http\TransferBuilder;
use Magento\Payment\Gateway\Http\TransferFactoryInterface;
use Magento\Payment\Gateway\Http\TransferInterface;

class TransferFactory implements TransferFactoryInterface
{
    /**
     * @var TransferBuilder
     */
    private $transferBuilder;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var Json
     */
    private $serializer;

    /**
     * @param TransferBuilder $transferBuilder
     * @param Config $config
     * @param Json $serializer
     */
    public function __construct(
        TransferBuilder $transferBuilder,
        Config          $config,
        Json            $serializer
    )
    {
        $this->transferBuilder = $transferBuilder;
        $this->config = $config;
        $this->serializer = $serializer;
    }

    /**
     * Builds gateway transfer object
     *
     * @param array $request
     * @return TransferInterface
     * @throws LocalizedException
     */
    public function create(array $request): TransferInterface
    {
        $request = [
            'headers' => $this->getHeaders(),
            'body' => $this->serializer->serialize($request),
        ];
        return $this->transferBuilder
            ->setBody($request)
            ->build();
    }

    /**
     * @return array
     * @throws LocalizedException
     */
    protected function getHeaders(): array
    {
        return [
            'Content-Type' => 'application/json',
            'Accept' => 'application/json;version=2',
            'X-Request-ID' => $this->config->getUuid(),
        ];
    }
}
