<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway\Request;

use ConnectPay\Payment\Gateway\Config\Config;
use ConnectPay\Payment\Gateway\SubjectReader;

class PaymentDataBuilder extends AbstractDataBuilder
{
    const PROVIDER_ID = 'connectpay_payment_provider';

    /**
     * @var Config
     */
    private $config;

    /**
     * @param Config $config
     * @param SubjectReader $subjectReader
     */
    public function __construct(
        Config        $config,
        SubjectReader $subjectReader
    )
    {
        $this->config = $config;
        parent::__construct($subjectReader);
    }

    /**
     * @param array $buildSubject
     * @return array[]
     */
    public function build(array $buildSubject): array
    {
        $paymentDO = $this->subjectReader->readPayment($buildSubject);
        $order = $paymentDO->getOrder();
        $payment = $paymentDO->getPayment();
        $providerId = $payment->getAdditionalInformation(self::PROVIDER_ID);
        $total = (float)$order->getGrandTotalAmount();
        return [
            Config::KEY_PAYMENT => [
                Config::KEY_PAYMENT_METHOD => Config::PAYMENT_METHOD,
                Config::KEY_PROVIDER_COUNTRY_CODE => $this->config->getProviderCountryCode(),
                Config::KEY_PROVIDER_ID => $providerId,
                Config::KEY_INSTRUCTED_AMOUNT => [
                    Config::KEY_AMOUNT => number_format($total, 2, '.', ''),
                    Config::KEY_CURRENCY => $order->getCurrencyCode(),
                ],
                Config::KEY_REMITTANCE_INFORMATION_UNSTRUCTURED => (string)$order->getId(),
            ]
        ];
    }
}
