<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway\Response;

use ConnectPay\Payment\Gateway\Config\Config;
use ConnectPay\Payment\Gateway\SubjectReader;
use Magento\Framework\DB\Transaction;
use Magento\Framework\Exception\LocalizedException;
use Magento\Payment\Gateway\Response\HandlerInterface;
use Magento\Sales\Api\Data\TransactionInterface;
use Magento\Sales\Api\OrderRepositoryInterface as OrderRepository;
use Magento\Sales\Api\TransactionRepositoryInterface as TransactionRepository;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\InvoiceRepository;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface as TransactionBuilder;
use Magento\Sales\Model\Service\InvoiceService;

class GetPaymentDetailsHandler implements HandlerInterface
{
    /**
     * @var InvoiceService
     */
    private $invoiceService;

    /**
     * @var SubjectReader
     */
    private $subjectReader;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    /**
     * @var InvoiceRepository
     */
    private $invoiceRepository;

    /**
     * @var Transaction
     */
    private $transaction;

    /**
     * @var TransactionBuilder
     */
    private $transactionBuilder;

    /**
     * @var TransactionRepository
     */
    private $transactionRepository;

    public function __construct(
        InvoiceService        $invoiceService,
        SubjectReader         $subjectReader,
        OrderRepository       $orderRepository,
        InvoiceRepository     $invoiceRepository,
        Transaction           $transaction,
        TransactionBuilder    $transactionBuilder,
        TransactionRepository $transactionRepository
    )
    {
        $this->invoiceService = $invoiceService;
        $this->subjectReader = $subjectReader;
        $this->orderRepository = $orderRepository;
        $this->invoiceRepository = $invoiceRepository;
        $this->transaction = $transaction;
        $this->transactionBuilder = $transactionBuilder;
        $this->transactionRepository = $transactionRepository;
    }

    /**
     * @param array $handlingSubject
     * @param array $response
     * @return void
     * @throws LocalizedException
     */
    public function handle(array $handlingSubject, array $response)
    {
        $statusArray = $this->subjectReader->readStatus($response);
        if (array_key_exists(Config::KEY_STATUS_IS_TERMINAL, $statusArray)) {
            $isTerminal = $statusArray[Config::KEY_STATUS_IS_TERMINAL];
            if ($isTerminal) {
                if (array_key_exists(Config::KEY_STATUS_CODE, $statusArray)) {
                    $statusCode = $statusArray[Config::KEY_STATUS_CODE];
                    $paymentDO = $this->subjectReader->readPayment($handlingSubject);
                    $orderId = $paymentDO->getOrder()->getId();
                    $order = $this->orderRepository->get($orderId);
                    $payment = $order->getPayment();
                    $cpPaymentId = $payment->getData(Config::KEY_PAYMENT_ID);
                    /** @var Order $order */


                    if ($statusCode === Config::APPROVED_STATUS) {
                        $invoice = $this->invoiceService->prepareInvoice($order);
                        $invoice->setTransactionId($cpPaymentId);
                        $invoice->register();
                        $invoice->pay();
                        $invoice->getOrder()->setState(Order::STATE_PROCESSING)
                            ->setStatus($order->getConfig()->getStateDefaultStatus(Order::STATE_PROCESSING));
                        $this->invoiceRepository->save($invoice);

                        $transactionSave = $this->transaction->addObject(
                            $invoice
                        )->addObject(
                            $invoice->getOrder()
                        );
                        $transactionSave->save();

                        $transaction = $this->transactionBuilder->setPayment($payment)
                            ->setOrder($order)
                            ->setTransactionId($cpPaymentId)
                            ->setFailSafe(true)
                            ->build(TransactionInterface::TYPE_CAPTURE);

                        $formattedAmount = $order->getBaseCurrency()->formatTxt($invoice->getBaseGrandTotal());
                        if ($order->getBaseCurrencyCode() != $order->getOrderCurrencyCode()) {
                            $formattedAmount = $formattedAmount . ' [' . $order->formatPriceTxt($order->getBaseGrandTotal()) . ']';
                        }

                        $message = 'Captured amount of %1.';
                        $message = __($message, $formattedAmount);
                        $payment->addTransactionCommentsToOrder($transaction, $message);

                        $this->orderRepository->save($order);
                        $this->transactionRepository->save($transaction);
                    } else {
                        $order->setState(Order::STATE_CANCELED)
                            ->setStatus($order->getConfig()->getStateDefaultStatus(Order::STATE_CANCELED));
                    }
                    $this->orderRepository->save($order);
                }
            }
        }
    }
}
