<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway\Validator;

use ConnectPay\Payment\Gateway\SubjectReader;
use Magento\Payment\Gateway\Validator\AbstractValidator;
use Magento\Payment\Gateway\Validator\ResultInterface;
use Magento\Payment\Gateway\Validator\ResultInterfaceFactory;

class InitiatePaymentResponseValidator extends AbstractValidator
{
    /**
     * @var SubjectReader
     */
    private $subjectReader;

    public function __construct(
        SubjectReader          $subjectReader,
        ResultInterfaceFactory $resultFactory
    )
    {
        $this->subjectReader = $subjectReader;
        parent::__construct($resultFactory);
    }

    /**
     * @param array $validationSubject
     * @return ResultInterface
     */
    public function validate(array $validationSubject): ResultInterface
    {
        $response = $this->subjectReader->readResponse($validationSubject);
        $isValid = $this->isValidResponse($response);
        return $this->createResult($isValid, !$isValid ? [__('Invalid Response')] : []);
    }

    private function isValidResponse(array $response): bool
    {
        if (!array_key_exists('payment', $response)) {
            return false;
        } else {
            $payment = $response['payment'];
            return array_key_exists('paymentId', $payment) && array_key_exists('redirectUrl', $payment);
      }
    }
}
