<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Model\Config\Source;

use ConnectPay\Payment\Gateway\Config\Config;
use Magento\Directory\Model\CountryFactory;
use Magento\Framework\Data\OptionSourceInterface;

class Country implements OptionSourceInterface
{
    /**
     * @var CountryFactory
     */
    private $countryFactory;

    /**
     * @var Config
     */
    private $config;

    /**
     * @param CountryFactory $countryFactory
     * @param Config $config
     */
    public function __construct(
        CountryFactory $countryFactory,
        Config         $config
    )
    {
        $this->countryFactory = $countryFactory;
        $this->config = $config;
    }

    /**
     * @return string[]
     */
    public function toOptionArray(): array
    {
        $optionArray = [];
        $data = Config::COUNTRY_CODES_ARRAY;

        foreach ($data as $value) {
            $optionArray[] = [
                'value' => $value, 'label' => __($this->getCountryName($value))
            ];
        }

        return $optionArray;
    }

    /**
     * @param string $countryCode
     * @return string
     */
    private function getCountryName(string $countryCode): string
    {
        $country = $this->countryFactory->create()->loadByCode($countryCode);
        return $country->getName();
    }

}
