<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Model\Request;

use ConnectPay\Payment\Gateway\Config\Config;
use ConnectPay\Payment\Gateway\Http\Client\Guzzle;
use ConnectPay\Payment\Gateway\Http\TransferFactory;
use ConnectPay\Payment\Model\ProviderMapper;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\Webapi\Rest\Request;
use Exception;
use Psr\Log\LoggerInterface as Logger;

class Provider
{
    /**
     * @var Guzzle
     */
    private $client;

    /**
     * @var Json
     */
    private $serializer;

    /**
     * @var ProviderMapper
     */
    private $providerMapper;

    /**
     * @var Logger
     */
    private $logger;

    /**
     * @var TransferFactory
     */
    private $transferFactory;

    /**
     * @param Guzzle $client
     * @param Json $serializer
     * @param ProviderMapper $providerMapper
     * @param Logger $logger
     * @param TransferFactory $transferFactory
     */
    public function __construct(
        Guzzle          $client,
        Json            $serializer,
        ProviderMapper  $providerMapper,
        Logger          $logger,
        TransferFactory $transferFactory
    )
    {
        $this->client = $client;
        $this->serializer = $serializer;
        $this->providerMapper = $providerMapper;
        $this->logger = $logger;
        $this->transferFactory = $transferFactory;
    }

    /**
     * @param $countryCode
     * @return array
     * @throws LocalizedException
     */
    public function getProviders($countryCode): array
    {
        $transferObject = $this->transferFactory->create([]);
        $data = $transferObject->getBody();

        $providersData = [];
        $data['query'] = [
            Config::KEY_COUNTRY_CODE => $countryCode,
        ];
        try {
            $response = $this->client->doRequest($data, Request::HTTP_METHOD_GET, Config::API_REQUEST_GET_PROVIDERS_ENDPOINT);
            $providersData = $this->serializer->unserialize($response->getBody()->getContents());
        } catch (Exception $exception) {
            $this->logger->debug($exception);
        }

        return $providersData;
    }
}

